@@ -1,12 +1,12 @@
 # -*- Mode: cperl; cperl-indent-level: 4 -*-
-# $Id: Straps.pm,v 1.17 2003/04/03 17:47:25 andy Exp $
+# $Id: Straps.pm,v 1.23 2003/11/01 05:57:00 andy Exp $
 
 package Test::Harness::Straps;
 
 use strict;
 use vars qw($VERSION);
 use Config;
-$VERSION = '0.14';
+$VERSION = '0.17_02';
 
 use Test::Harness::Assert;
 use Test::Harness::Iterator;
@@ -58,11 +58,9 @@ The interface is currently incomplete.  I<Please> contact the author
 if you'd like a feature added or something change or just have
 comments.
 
-=head2 Construction
+=head1 Construction
 
-=over 4
-
-=item B<new>
+=head2 C<new>
 
   my $strap = Test::Harness::Straps->new;
 
@@ -80,9 +78,7 @@ sub new {
     return $self;
 }
 
-=begin _private
-
-=item B<_init>
+=head2 C<_init>
 
   $strap->_init;
 
@@ -93,27 +89,23 @@ Initialize the internal state of a strap to make it ready for parsing.
 sub _init {
     my($self) = shift;
 
-    $self->{_is_vms}   = $^O eq 'VMS';
-    $self->{_is_win32} = $^O eq 'Win32';
+    $self->{_is_vms}   = ( $^O eq 'VMS' );
+    $self->{_is_win32} = ( $^O =~ /^(MS)?Win32$/ );
+    $self->{_is_macos} = ( $^O eq 'MacOS' );
 }
 
-=end _private
+=head1 Analysis
 
-=back
-
-=head2 Analysis
-
-=over 4
-
-=item B<analyze>
+=head2 C<analyze>
 
   my %results = $strap->analyze($name, \@test_output);
 
-Analyzes the output of a single test, assigning it the given $name for
-use in the total report.  Returns the %results of the test.  See
-L<Results>.
+Analyzes the output of a single test, assigning it the given C<$name>
+for use in the total report.  Returns the C<%results> of the test.
+See L<Results>.
 
-@test_output should be the raw output from the test, including newlines.
+C<@test_output> should be the raw output from the test, including
+newlines.
 
 =cut
 
@@ -231,7 +223,7 @@ sub _analyze_line {
     $self->{'next'} = $result{number} + 1 if $type eq 'test';
 }
 
-=item B<analyze_fh>
+=head2 C<analyze_fh>
 
   my %results = $strap->analyze_fh($name, $test_filehandle);
 
@@ -246,11 +238,11 @@ sub analyze_fh {
     $self->_analyze_iterator($name, $it);
 }
 
-=item B<analyze_file>
+=head2 C<analyze_file>
 
   my %results = $strap->analyze_file($test_file);
 
-Like C<analyze>, but it runs the given $test_file and parses it's
+Like C<analyze>, but it runs the given C<$test_file> and parses its
 results.  It will also use that name for the total report.
 
 =cut
@@ -308,9 +300,7 @@ else {
 }
 
 
-=begin _private
-
-=item B<_switches>
+=head2 C<_switches>
 
   my $switches = $self->_switches($file);
 
@@ -321,35 +311,33 @@ Formats and returns the switches necessary to run the test.
 sub _switches {
     my($self, $file) = @_;
 
+    my @switches = grep { defined } ($Test::Harness::Switches, $ENV{HARNESS_PERL_SWITCHES});
+
     local *TEST;
     open(TEST, $file) or print "can't open $file. $!\n";
-    my $first = <TEST>;
-    my $s = $Test::Harness::Switches || '';
-    $s .= " $ENV{'HARNESS_PERL_SWITCHES'}"
-      if exists $ENV{'HARNESS_PERL_SWITCHES'};
-
-    if ($first =~ /^#!.*\bperl.*\s-\w*([Tt]+)/) {
-        # When taint mode is on, PERL5LIB is ignored.  So we need to put
-        # all that on the command line as -Is.
-        $s .= join " ", qq[ "-$1"], map {qq["-I$_"]} $self->_filtered_INC;
-    }
-    elsif ($^O eq 'MacOS') {
-        # MacPerl's putenv is broken, so it will not see PERL5LIB.
-        $s .= join " ", map {qq["-I$_"]} $self->_filtered_INC;
-    }
-
+    my $shebang = <TEST>;
     close(TEST) or print "can't close $file. $!\n";
 
-    return $s;
+    if ( $shebang =~ /^#!.*\bperl.*\s-\w*([Tt]+)/ ) {
+        # When taint mode is on, PERL5LIB is ignored.  So we need to put
+        # all that on the command line as -Is.
+	push @switches, map qq["-$1"], map {qq["-I$_"]} $self->_filtered_INC;
+    }
+    elsif ( $self->{_is_macos} ) {
+        # MacPerl's putenv is broken, so it will not see PERL5LIB.
+	push @switches, map {qq["-I$_"]} $self->_filtered_INC;
+    }
+
+    return join( " ", @switches );
 }
 
 
-=item B<_INC2PERL5LIB>
+=head2 C<_INC2PERL5LIB>
 
   local $ENV{PERL5LIB} = $self->_INC2PERL5LIB;
 
-Takes the current value of @INC and turns it into something suitable
-for putting onto PERL5LIB.
+Takes the current value of C<@INC> and turns it into something suitable
+for putting onto C<PERL5LIB>.
 
 =cut
 
@@ -361,12 +349,12 @@ sub _INC2PERL5LIB {
     return join $Config{path_sep}, $self->_filtered_INC;
 }
 
-=item B<_filtered_INC>
+=head2 C<_filtered_INC>
 
   my @filtered_inc = $self->_filtered_INC;
 
-Shortens @INC by removing redundant and unnecessary entries.
-Necessary for OS's with limited command line lengths, like VMS.
+Shortens C<@INC> by removing redundant and unnecessary entries.
+Necessary for OSes with limited command line lengths, like VMS.
 
 =cut
 
@@ -374,22 +362,28 @@ sub _filtered_INC {
     my($self, @inc) = @_;
     @inc = @INC unless @inc;
 
-    # VMS has a 255-byte limit on the length of %ENV entries, so
-    # toss the ones that involve perl_root, the install location
-    # for VMS
     if( $self->{_is_vms} ) {
+	# VMS has a 255-byte limit on the length of %ENV entries, so
+	# toss the ones that involve perl_root, the install location
         @inc = grep !/perl_root/i, @inc;
+
+    } elsif ( $self->{_is_win32} ) {
+	# Lose any trailing backslashes in the Win32 paths
+	s/[\\\/+]$// foreach @inc;
     }
 
+    my %dupes;
+    @inc = grep !$dupes{$_}++, @inc;
+
     return @inc;
 }
 
 
-=item B<_restore_PERL5LIB>
+=head2 C<_restore_PERL5LIB>
 
   $self->_restore_PERL5LIB;
 
-This restores the original value of the PERL5LIB environment variable.
+This restores the original value of the C<PERL5LIB> environment variable.
 Necessary on VMS, otherwise a no-op.
 
 =cut
@@ -404,26 +398,16 @@ sub _restore_PERL5LIB {
     }
 }
 
-
-=end _private
-
-=back
-
-
-=begin _private
-
-=head2 Parsing
+=head1 Parsing
 
 Methods for identifying what sort of line you're looking at.
 
-=over 4
-
-=item B<_is_comment>
+=head2 C<_is_comment>
 
   my $is_comment = $strap->_is_comment($line, \$comment);
 
 Checks if the given line is a comment.  If so, it will place it into
-$comment (sans #).
+C<$comment> (sans #).
 
 =cut
 
@@ -439,14 +423,14 @@ sub _is_comment {
     }
 }
 
-=item B<_is_header>
+=head2 C<_is_header>
 
   my $is_header = $strap->_is_header($line);
 
-Checks if the given line is a header (1..M) line.  If so, it places
-how many tests there will be in $strap->{max}, a list of which tests
-are todo in $strap->{todo} and if the whole test was skipped
-$strap->{skip_all} contains the reason.
+Checks if the given line is a header (1..M) line.  If so, it places how
+many tests there will be in C<< $strap->{max} >>, a list of which tests
+are todo in C<< $strap->{todo} >> and if the whole test was skipped
+C<< $strap->{skip_all} >> contains the reason.
 
 =cut
 
@@ -483,12 +467,12 @@ sub _is_header {
     }
 }
 
-=item B<_is_test>
+=head2 C<_is_test>
 
   my $is_test = $strap->_is_test($line, \%test);
 
 Checks if the $line is a test report (ie. 'ok/not ok').  Reports the
-result back in %test which will contain:
+result back in C<%test> which will contain:
 
   ok            did it succeed?  This is the literal 'ok' or 'not ok'.
   name          name of the test (if any)
@@ -498,7 +482,7 @@ result back in %test which will contain:
   reason        why is it todo or skip? (if any)
 
 If will also catch lone 'not' lines, note it saw them 
-$strap->{saw_lone_not} and the line in $strap->{lone_not_line}.
+C<< $strap->{saw_lone_not} >> and the line in C<< $strap->{lone_not_line} >>.
 
 =cut
 
@@ -522,8 +506,8 @@ sub _is_test {
 
     # We pulverize the line down into pieces in three parts.
     if( my($not, $num, $extra)    = $line  =~ /$Report_Re/ox ) {
-        my($name, $control) = split /(?:[^\\]|^)#/, $extra if $extra;
-        my($type, $reason)  = $control =~ /^\s*(\S+)(?:\s+(.*))?$/ if $control;
+        my ($name, $control) = $extra ? split(/(?:[^\\]|^)#/, $extra) : ();
+        my ($type, $reason)  = $control ? $control =~ /^\s*(\S+)(?:\s+(.*))?$/ : ();
 
         $test->{number} = $num;
         $test->{ok}     = $not ? 0 : 1;
@@ -541,7 +525,7 @@ sub _is_test {
         return $YES;
     }
     else{
-        # Sometimes the "not " and "ok" will be on seperate lines on VMS.
+        # Sometimes the "not " and "ok" will be on separate lines on VMS.
         # We catch this and remember we saw it.
         if( $line =~ /^not\s+$/ ) {
             $self->{saw_lone_not} = 1;
@@ -552,7 +536,7 @@ sub _is_test {
     }
 }
 
-=item B<_is_bail_out>
+=head2 C<_is_bail_out>
 
   my $is_bail_out = $strap->_is_bail_out($line, \$reason);
 
@@ -573,12 +557,12 @@ sub _is_bail_out {
     }
 }
 
-=item B<_reset_file_state>
+=head2 C<_reset_file_state>
 
   $strap->_reset_file_state;
 
-Resets things like $strap->{max}, $strap->{skip_all}, etc... so its
-ready to parse the next file.
+Resets things like C<< $strap->{max} >> , C<< $strap->{skip_all} >>,
+etc. so it's ready to parse the next file.
 
 =cut
 
@@ -595,14 +579,10 @@ sub _reset_file_state {
     $self->{'next'}       = 1;
 }
 
-=back
+=head1 Results
 
-=end _private
-
-
-=head2 Results
-
-The %results returned from analyze() contain the following information:
+The C<%results> returned from C<analyze()> contain the following
+information:
 
   passing           true if the whole test is considered a pass 
                     (or skipped), false if its a failure
@@ -643,17 +623,13 @@ There is one final item, the details.
 Element 0 of the details is test #1.  I tried it with element 1 being
 #1 and 0 being empty, this is less awkward.
 
-=begin _private
-
-=over 4
-
-=item B<_detailize>
+=head2 C<_detailize>
 
   my %details = $strap->_detailize($pass, \%test);
 
-Generates the details based on the last test line seen.  $pass is true
-if it was considered to be a passed test.  %test is the results of the
-test you're summarizing.
+Generates the details based on the last test line seen.  C<$pass> is
+true if it was considered to be a passed test.  C<%test> is the results
+of the test you're summarizing.
 
 =cut
 
@@ -677,17 +653,14 @@ sub _detailize {
     return %details;
 }
 
-=back
-
-=end _private
-
 =head1 EXAMPLES
 
 See F<examples/mini_harness.plx> for an example of use.
 
 =head1 AUTHOR
 
-Michael G Schwern E<lt>schwern@pobox.comE<gt>
+Michael G Schwern C<< <schwern@pobox.com> >>, currently maintained by
+Andy Lester C<< <andy@petdance.com> >>.
 
 =head1 SEE ALSO
 
